struct PSInput
{
	float4 position : SV_POSITION;
	float2 uv : TEXCOORD;
	float4 color : COLOR;
	float3 normal : NORMAL;
	float4 worldPosition : POSITION;
	float4 worldVertex : WORLDVERTEX;
};

Texture2D g_texture : register(t0);
SamplerState g_sampler : register(s0);

cbuffer cb0 : register(b0)
{
	float4x4 mvMat;
	float4x4 projMat;
	float4x4 modelMat;
	float3x3 inverseModelMat;
	float3 lightDirection;
	int highlight;
	float3 spotPosition;
	float3 lightMultiplier;
};

PSInput VSMain(float3 position : POSITION, float2 uv : TEXCOORD, float3 color : COLOR, float3 normal: NORMAL)
{
	PSInput result;

	float4x4 finalMatrix = mul(projMat, mvMat);
	result.position = mul(finalMatrix, float4(position, 1.0));
	result.worldPosition = mul(mvMat, float4(position, 1.0));
	result.uv = uv;
	result.color = float4(color, 1.0);

	float3x3 normalMatrix = transpose(inverseModelMat);
	result.normal = normalize(mul(normalMatrix, normal));

	result.worldVertex = mul(modelMat, float4(position, 1.0));

	return result;
}

float4 PSMain(PSInput input) : SV_TARGET
{
	float4 textureColor = input.color * g_texture.Sample(g_sampler, input.uv);
	if(textureColor.a < 0.3)
		discard;

	float4 fragColor = textureColor;

	float3 ambient = 0.1 * fragColor.rgb;
	float diffuseCoefficient = max(0.0, dot(input.normal, -lightDirection.xyz));
	float3 diffuse = diffuseCoefficient * fragColor.rgb;

	fragColor.rgb = ambient+diffuse;//lerp(ambient+diffuse, ambient, clamp(-input.worldVertex.z/20.0, 0.0, 1.0));

	fragColor *= float4(lightMultiplier,1);

	//POINT LIGHT
	{
		float3 spotDirection = float3(0.0, 0.0, -1.0);
		float3 surfaceToLight = normalize(spotPosition - input.worldVertex.xyz);
		float distanceToLight = length(spotPosition - input.worldVertex.xyz);
		float diffuseCoefficient = max(0.0, dot(input.normal, surfaceToLight));
		float lightAttenuation = 0.01;
		float3 lightIntensity = float3(0.3, 0.3, 0.3);
		float attenuation = 1.0 / (1.0 + lightAttenuation * pow(distanceToLight, 2));

		float spotExteriorAngle = 60.0;
		float spotInteriorAngle = 40.0;
		float lightToSurfaceAngle = acos(dot(-surfaceToLight, normalize(spotDirection))) / 0.017453292519943;
		if(lightToSurfaceAngle > spotInteriorAngle)
		{
			float attenuationFactor = clamp(lerp(0.0, 1.0, (lightToSurfaceAngle-spotInteriorAngle)/(spotExteriorAngle-spotInteriorAngle)), 0.0, 1.0);
			attenuationFactor = 1.0 - attenuationFactor;
			attenuation *= attenuationFactor;
		}

		fragColor.rgb += lightIntensity*diffuseCoefficient*attenuation * textureColor.rgb;
	}
	//END POINT LIGHT

	float blendFactor = clamp(length(input.worldPosition)/80.0, 0.0, 1.0);

	float4 outColor;
	outColor = fragColor;

	//float gammaValue = 2.2;
	//float3 gamma = float3(1.0/gammaValue, 1.0/gammaValue, 1.0/gammaValue);
	outColor = float4(outColor.rgb, 1.0);//float4(pow(outColor.rgb, gamma), 1.0);
	if(highlight == 1)
	{
		//outColor.a = 2.0;
		//Tmp before we get a real highlight shader:
		outColor.rgb = textureColor.rgb;
	}

	return outColor;
}
